@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -30,7 +30,7 @@
 #include <string>
 #include <map>
 #include <set>
-#include <iosfwd>
+#include <iostream>
 
 namespace TiCC {
   class Configuration {
@@ -38,6 +38,7 @@ namespace TiCC {
     typedef std::map<std::string, ssMap> sssMap;
   public:
     Configuration();
+    void merge( const Configuration&, bool = false );
     bool fill( const std::string& );
     bool fill( const std::string&, const std::string& );
     bool hasSection( const std::string& ) const;
@@ -59,7 +60,7 @@ namespace TiCC {
     std::string configDir() const { return lookUp( "configDir" ); };
   private:
     sssMap myMap;
-    std::string cdir; // not used. left here for binary compatibility
+    bool get_att_val( const std::string&, const std::string& );
   };
 
   inline std::ostream& operator<<( std::ostream& os, const Configuration& c ){
