@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -25,6 +25,7 @@
 */
 
 #include <vector>
+#include <string>
 
 #ifndef TICC_FILE_UTILS_H
 #define TICC_FILE_UTILS_H
@@ -34,7 +35,7 @@ namespace TiCC {
   std::vector<std::string> searchFilesExt( const std::string&,
 					   const std::string&,
 					   bool = true );
-  std::vector<std::string> searchFiles( const std::string& name,
+  inline std::vector<std::string> searchFiles( const std::string& name,
 					bool recurse = true ){
     return searchFilesExt( name, "", recurse );
   }
