@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -55,10 +55,21 @@ namespace TiCC {
     return split_at_first_of( s, vec, " \r\t\n", exact );
   }
 
+  std::vector<std::string> split_at( const std::string&,
+				     const std::string&,
+				     size_t = 0 );
+  std::vector<std::string> split_at_first_of( const std::string&,
+					      const std::string&,
+					      size_t = 0 );
+  inline std::vector<std::string> split( const std::string& s,
+					 size_t num = 0 ){
+    return split_at_first_of( s, " \r\t\n", num );
+  }
+
   std::string format_nonascii( const std::string& );
 
   template< typename T >
-    T stringTo( const std::string& str ) {
+    inline T stringTo( const std::string& str ) {
     T result;
     std::stringstream dummy ( str );
     if ( !( dummy >> result ) ) {
@@ -81,7 +92,7 @@ namespace TiCC {
   }
 
   template< typename T >
-    bool stringTo( const std::string& str, T& result ) {
+    inline bool stringTo( const std::string& str, T& result ) {
     try {
       result = stringTo<T>( str );
       return true;
@@ -92,7 +103,7 @@ namespace TiCC {
   }
 
   template <typename T>
-    bool stringTo( const std::string& s, T &answer, T low, T upp ){
+    inline bool stringTo( const std::string& s, T &answer, T low, T upp ){
     try {
       T tmp = stringTo<T>( s );
       if ( (tmp >= low) && (tmp <= upp) ){
@@ -107,7 +118,7 @@ namespace TiCC {
   }
 
   template< typename T >
-    std::string toString ( const T& obj, bool=false ) {
+    inline std::string toString ( const T& obj, bool=false ) {
     std::stringstream dummy;
     if ( !( dummy << obj ) ) {
       throw( std::runtime_error( "conversion failed" ) );
@@ -117,6 +128,7 @@ namespace TiCC {
 
   std::string basename( const std::string& );
   std::string dirname( const std::string& );
+  std::string realpath( const std::string& );
 
 }
 
