@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -27,11 +27,7 @@
 #ifndef TICC_TAR_TOOLS_H
 #define TICC_TAR_TOOLS_H
 
-#ifdef HAVE_LIBTAR_H
 #include "libtar.h"
-#else
-#define TAR int
-#endif
 #include <fstream>
 #include <vector>
 
