@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -42,7 +42,7 @@ namespace Tries {
     friend std::ostream& operator<< LTGT ( std::ostream&,
 					   const TrieNode<Info> * );
   public:
-    TrieNode( char );
+    explicit TrieNode( char );
     ~TrieNode();
     Info *add_to_tree( Info *, const std::string& );
     Info *scan_tree( const std::string& ) const;
@@ -157,8 +157,12 @@ namespace Tries {
     // we can store the info.
     //
     if ( lab[0] == '\0') {
-      if ( !the_info )
+      if ( !the_info ){
 	the_info = info;
+      }
+      else {
+	delete( info );
+      }
       return the_info;
     }
     else {
