@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -27,6 +27,9 @@
 #ifndef TICC_ZIP_TOOLS_H
 #define TICC_ZIP_TOOLS_H
 
+#include <string>
+#include <iostream>
+
 namespace TiCC {
 
   bool bz2Compress( const std::string&, const std::string& = "" );
