@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -26,6 +26,8 @@
 
 */
 
+#include "ticcutils/CommandLine.h"
+
 #include <cassert>
 #include <string>
 #include <vector>
@@ -34,13 +36,17 @@
 #include <iostream>
 
 #include "ticcutils/StringOps.h"
-#include "ticcutils/CommandLine.h"
 #include "ticcutils/PrettyPrint.h"
 
 using namespace std;
 
 namespace TiCC {
 
+  class ImplementationError: public std::logic_error {
+  public:
+    explicit ImplementationError( const std::string& s ): std::logic_error( "CommandLine: implementation error." + s ){};
+  };
+
   CL_Options::CL_Options( const string& valid_s, const string& valid_l ):
     is_init(false),
     debug(false){
@@ -60,7 +66,7 @@ namespace TiCC {
 
   bool CL_Options::parse_args( const int argc, const char * const *argv ){
     if ( is_init ){
-      throw OptionError( "cannot parse() a commandline twice" );
+      throw ImplementationError( "cannot parse() a commandline twice" );
     }
     if ( Parse_Command_Line( argc, argv ) )
       is_init = true;
@@ -69,7 +75,7 @@ namespace TiCC {
 
   bool CL_Options::parse_args( const std::string& args ){
     if ( is_init ){
-      throw OptionError( "cannot parse() a commandline twice" );
+      throw ImplementationError( "cannot parse() a commandline twice" );
     }
     const char *argstr = args.c_str();
     if ( Parse_Command_Line( 0, &argstr ) )
@@ -419,7 +425,7 @@ namespace TiCC {
       default:
 	argument.stat = MASS;
 	if ( debug ){
-	  cerr << "OPTION=" << Option << endl;
+	  cerr << "MASS-OPTION=" << Option << endl;
 	}
 	argument.val = Option;
 	arguments.push_back(argument);
@@ -435,7 +441,7 @@ namespace TiCC {
     bool doCheck = !valid_long.empty() || !valid_chars.empty();
     if ( doCheck ){
       if ( debug ){
-	cerr << "we must check those " << endl;
+	cerr << "we must check this list" << endl;
       }
       auto it = arguments.begin();
       while ( it != arguments.end() ){
@@ -461,8 +467,6 @@ namespace TiCC {
 	      if ( debug ){
 		cerr << "no parameter: OK" << endl;
 	      }
-	      ++it;
-	      continue;
 	    }
 	    else {
 	      if ( debug ){
@@ -481,8 +485,6 @@ namespace TiCC {
 		  if ( debug ){
 		    cerr << "OK: " << it->val << endl;
 		  }
-		  ++it;
-		  continue;
 		}
 	      }
 	      else if ( !has_opt ){
@@ -491,8 +493,6 @@ namespace TiCC {
 		}
 		throw OptionError( "missing value for long option: '"
 				   + it->s + "'" );
-		++it;
-		continue;
 	      }
 	    }
 	  }
@@ -500,24 +500,16 @@ namespace TiCC {
 	    if ( debug ){
 	      cerr << "found a parameter: " <<  it->val << endl;
 	    }
-	    ++it;
-	    continue;
 	  }
 	  else {
-	    auto next = it+1;
-	    while ( next != arguments.end() && next->stat == MASS ){
-	      ++next;
+	    // ok, some random data. assume it to be a Mass opt
+	    arg a = *it;
+	    a.stat = MASS;
+	    it->val.clear();
+	    it = arguments.insert( ++it, a );
+	    if ( debug ){
+	      cerr << "inserted " << a << endl;
 	    }
-	    if ( next != arguments.end() ){
-	      throw OptionError( string("option '") + it->s
-				 + "' may not have a value. (found "
-				 + it->val + ")" );
-	    }
-	    else {
-	      MassOpts.push_back( it->val );
-	      it->val.clear();
-	    }
-	    ++it;
 	  }
 	}
 	else if ( it->stat == MIN || it->stat == PLUS ){
@@ -539,8 +531,6 @@ namespace TiCC {
 	      if ( debug ){
 		cerr << "no parameter: OK" << endl;
 	      }
-	      ++it;
-	      continue;
 	    }
 	    else {
 	      if ( debug ){
@@ -559,8 +549,6 @@ namespace TiCC {
 		  if ( debug ){
 		    cerr << "OK: " << it->val << endl;
 		  }
-		  ++it;
-		  continue;
 		}
 	      }
 	      else if ( !has_opt ){
@@ -570,50 +558,28 @@ namespace TiCC {
 		throw OptionError( string("missing value for option ''")
 				   + it->c + "'" );
 	      }
-	      else {
-		++it;
-		continue;
-	      }
 	    }
 	  }
 	  else if ( has_par || has_opt ){
 	    if ( debug ){
 	      cerr << "found a parameter: " <<  it->val << endl;
 	    }
-	    ++it;
-	    continue;
 	  }
 	  else {
-	    auto next = it+1;
-	    while ( next != arguments.end() && next->stat == MASS ){
-	      ++next;
+	    // ok, some random data. assume it to be a Mass opt
+	    arg a = *it;
+	    a.stat = MASS;
+	    it->val.clear();
+	    it = arguments.insert( ++it, a );
+	    if ( debug ){
+	      cerr << "inserted " << a << endl;
 	    }
-	    if ( next != arguments.end() ){
-	      throw OptionError( string("option '") + it->c
-				 + "' may not have a value. (found "
-				 + it->val + ")" );
-	    }
-	    else {
-	      MassOpts.push_back( it->val );
-	      it->val.clear();
-	    }
-	    ++it;
 	  }
 	}
 	else if ( it->stat == MASS ){
-	  auto next = it+1;
-	  while ( next != arguments.end() && next->stat == MASS ){
-	    ++next;
-	  }
-	  if ( next != arguments.end() ){
-	    throw OptionError( "spurious value: '" + it->val +
-			       "' in options" );
-	  }
-	  else {
-	    MassOpts.push_back( it->val );
-	    it = arguments.erase(it);
-	  }
+	  // skip
 	}
+	++it;
       }
     }
     if ( debug ){
@@ -632,11 +598,9 @@ namespace TiCC {
 	MassOpts.push_back( it.val );
       }
     }
-    if ( debug ){
-      cerr << "mass opts: " << MassOpts << endl;
-    }
 
     if ( debug ){
+      cerr << "Final mass opts: " << MassOpts << endl;
       cerr << "CL_options after Parse " << endl;
       dump(cerr);
       cerr << endl;
@@ -675,8 +639,7 @@ namespace TiCC {
   }
 
   void CL_Options::set_long_options( const string& s ){
-    vector<string> parts;
-    TiCC::split_at( s, parts, "," );
+    vector<string> parts = TiCC::split_at( s, "," );
     for ( auto value: parts ){
       string::size_type pos = value.find( ':' );
       if ( pos != string::npos ){
@@ -686,7 +649,7 @@ namespace TiCC {
 	    valid_long_opt.insert( value );
 	  }
 	  else {
-	    throw OptionError( "':' may only be present at the end of a long option ("
+	    throw ImplementationError( "':' may only be present at the end of a long option specification ("
 			       + value + ")" );
 	  }
 	}
@@ -695,8 +658,8 @@ namespace TiCC {
 	  valid_long_par.insert( value );
 	}
 	else {
-	  throw OptionError( "':' may only be present at the end of a long option ("
-			     + value + ")" );
+	  throw ImplementationError( "':' may only be present at the end of a long option specification ("
+				     + value + ")" );
 	}
       }
       valid_long.insert( value );
