@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -25,6 +25,8 @@
 
 */
 
+#include "ticcutils/Configuration.h"
+
 #include <string>
 #include <map>
 #include <set>
@@ -32,7 +34,6 @@
 #include <stdexcept>
 #include <iostream>
 #include "ticcutils/StringOps.h"
-#include "ticcutils/Configuration.h"
 
 using namespace std;
 
@@ -87,6 +88,22 @@ namespace TiCC {
     return result;
   }
 
+  bool Configuration::get_att_val( const string& line, const string& section ){
+    string::size_type pos = line.find("=");
+    if ( pos != string::npos ){
+      string att = line.substr(0,pos);
+      att = TiCC::trim(att);
+      string val = line.substr(pos+1);
+      val = TiCC::trim(val);
+      if ( val[0] == '"' && val[val.length()-1] == '"' )
+	val = val.substr(1, val.length()-2);
+      val = fixControls( val );
+      myMap[section][att] = val;
+      return true;
+    }
+    return false;
+  }
+
   bool Configuration::fill( const string& fileName ){
     ifstream is( fileName );
     if ( !is ){
@@ -110,7 +127,7 @@ namespace TiCC {
 	continue;
       if ( line[0] == '#' )
 	continue;
-      if ( line.find( "[[") == 0  )
+      if ( line.find( "[[" ) == 0  )
 	if ( line[line.length()-1] == ']' &&
 	     line[line.length()-2] == ']' ){
 	  section = line.substr(2,line.length()-4);
@@ -121,19 +138,9 @@ namespace TiCC {
 	  return false;
 	}
       else {
-	string::size_type pos = line.find("=");
-	if ( pos != string::npos ){
-	  string att = line.substr(0,pos);
-	  att = TiCC::trim(att);
-	  string val = line.substr(pos+1);
-	  val = TiCC::trim(val);
-	  if ( val[0] == '"' && val[val.length()-1] == '"' )
-	    val = val.substr(1, val.length()-2);
-	  val = fixControls( val );
-	  myMap[section][att] = val;
-	}
-	else {
-	  cerr << "invalid attribute value pair in line '" << line << "'" << endl;
+	if ( !get_att_val ( line, section ) ){
+	  cerr << "invalid attribute value pair in line '"
+	       << line << "'" << endl;
 	  return false;
 	}
       }
@@ -158,7 +165,7 @@ namespace TiCC {
 	continue;
       if ( line[0] == '#' )
 	continue;
-      if ( line.find( "[[") == 0  )
+      if ( line.find( "[[" ) == 0  )
 	if ( line[line.length()-1] == ']' &&
 	     line[line.length()-2] == ']' ){
 	  localsection = line.substr(2,line.length()-4);
@@ -171,18 +178,9 @@ namespace TiCC {
 	}
       else if ( localsection == section ){
 	found = true;
-	string::size_type pos = line.find("=");
-	if ( pos != string::npos ){
-	  string att = line.substr(0,pos);
-	  att = TiCC::trim(att);
-	  string val = line.substr(pos+1);
-	  val = TiCC::trim(val);
-	  if ( val[0] == '"' && val[val.length()-1] == '"' )
-	    val = val.substr(1, val.length()-2);
-	  myMap[section][att] = val;
-	}
-	else {
-	  cerr << "invalid attribute value pair in line '" << line << "'" << endl;
+	if ( !get_att_val( line, section ) ){
+	  cerr << "invalid attribute value pair in line '"
+	       << line << "'" << endl;
 	  return false;
 	}
       }
@@ -195,6 +193,23 @@ namespace TiCC {
     return true;
   }
 
+  string encode_ctrl( const string& in ){
+    string out;
+    for ( const auto& c : in ){
+      switch ( c ){
+      case '\n': out += "\\n";
+	break;
+      case '\r': out += "\\r";
+	break;
+      case '\t': out += "\\t";
+	break;
+      default:
+	out += c;
+      }
+    }
+    return out;
+  }
+
   void Configuration::dump( ostream& os ) const {
     auto it1 = myMap.find("global");
     if ( it1 == myMap.end() ){
@@ -204,7 +219,6 @@ namespace TiCC {
     os << "[[global]]" << endl;
     auto it2 = it1->second.begin();
     while ( it2 != it1->second.end() ){
-      string out = it2->second;
       os << it2->first << "=" << it2->second << endl;
       ++it2;
     }
@@ -214,7 +228,7 @@ namespace TiCC {
 	os << endl << "[[" << it1->first << "]]" << endl;
 	it2 = it1->second.begin();
 	while ( it2 != it1->second.end() ){
-	  os << it2->first << "=" << it2->second << endl;
+	  os << it2->first << "=" << encode_ctrl(it2->second) << endl;
 	  ++it2;
 	}
       }
@@ -343,4 +357,21 @@ namespace TiCC {
     return false;
   }
 
+  void Configuration::merge( const Configuration& in, bool override ) {
+    // get al sections from in;
+    set<string> sections = in.lookUpSections();
+    for ( const auto& s : sections ){
+      // for every section, get all at-val pairs
+      auto avs = in.lookUpAll( s );
+      for ( const auto av : avs ){
+	// merge every at-val in the wanted section
+	if ( !override
+	     &&  myMap[s].find( av.first ) != myMap[s].end() ){
+	  continue;
+	}
+	setatt( av.first, av.second, s );
+      }
+    }
+  }
+
 }
