@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -25,6 +25,8 @@
 
 */
 
+#include "ticcutils/FileUtils.h"
+
 #include <cerrno>
 #include <cstring>
 #include <vector>
@@ -37,11 +39,15 @@
 #include <sys/stat.h>
 #include <dirent.h>
 #include "config.h"
-#ifdef HAVE_BOOST_REGEX
-#include <boost/regex.hpp>
+#if HAVE_BOOST_REGEX
+#  include <boost/regex.hpp>
+#  define regex boost::regex
+#  define regex_error boost::regex_error
+#  define regex_search boost::regex_search
+#else
+#  include <regex>
 #endif
 #include "ticcutils/StringOps.h"
-#include "ticcutils/FileUtils.h"
 
 using namespace std;
 namespace TiCC {
@@ -54,10 +60,11 @@ namespace TiCC {
       return result;
     }
     else if ( res != 0 ){
-      cerr << "TiCC::glob:" << strerror( errno ) << endl;
-      exit(EXIT_FAILURE);
+      string mess = "TiCC::glob: ";
+      mess += strerror( errno );
+      throw runtime_error( mess );
     }
-    for(unsigned int i=0;i<glob_result.gl_pathc;++i){
+    for( unsigned int i=0; i<glob_result.gl_pathc; ++i ){
       result.push_back(string(glob_result.gl_pathv[i]));
     }
     globfree(&glob_result);
@@ -86,8 +93,8 @@ namespace TiCC {
 		       vector<string>& result, bool recurse ){
     DIR *dir = opendir( dirName.c_str() );
     if ( !dir ){
-      cerr << "unable to open dir" << dirName << endl;
-      exit(EXIT_FAILURE);
+      string mess = "TiCC::gatherFilesExt: unable to open dir " + dirName;
+      throw runtime_error( mess );
     }
     struct dirent *entry = readdir( dir );
     while ( entry ){
@@ -120,21 +127,20 @@ namespace TiCC {
       return result;
     }
     else if ( !isDir( name ) ){
-      cerr << "the name '" << name
-	   << "' doesn't match a file or directory." << endl;
-      exit(EXIT_FAILURE);
+      string mess = "TiCC::searchFilesExt: the name '" + name
+	+ "' doesn't match a file or directory.";
+      throw runtime_error( mess );
     }
     gatherFilesExt( name, ext, result, recurse );
     return result;
   }
 
-#ifdef HAVE_BOOST_REGEX
-  void gatherFilesMatch( const string& dirName, const boost::regex& match,
+  void gatherFilesMatch( const string& dirName, const regex& match,
 			 vector<string>& result, bool recurse ){
     DIR *dir = opendir( dirName.c_str() );
     if ( !dir ){
-      cerr << "unable to open dir" << dirName << endl;
-      exit(EXIT_FAILURE);
+      string mess = "TiCC::gatherFilesMatch: unable to open dir " + dirName;
+      throw runtime_error( mess );
     }
     struct dirent *entry = readdir( dir );
     while ( entry ){
@@ -145,7 +151,7 @@ namespace TiCC {
 	  gatherFilesMatch( fullName, match, result, recurse );
 	}
       }
-      else if ( boost::regex_search( name, match ) ){
+      else if ( regex_search( name, match ) ){
 	result.push_back( fullName );
       }
       entry = readdir( dir );
@@ -156,8 +162,8 @@ namespace TiCC {
   static string wildToRegExp( const string& wild ){
     // convert 'shell'-like wildcards into a regexp
     string result;
-    for ( size_t i=0; i < wild.length(); ++i ){
-      switch( wild[i] ){
+    for ( const auto& c : wild ){
+      switch( c ){
       case '*':
 	result += ".*";
 	break;
@@ -165,11 +171,10 @@ namespace TiCC {
 	result += ".";
 	break;
       case '.':
-	result += "\\";
-	result += wild[i];
+	result += "\\.";
 	break;
       default:
-	result += wild[i];
+	result += c;
       }
     }
     //    cerr << "wild to regexp: " << wild << " ==> " << result << endl;
@@ -182,7 +187,7 @@ namespace TiCC {
     vector<string> result;
     string reg = wildToRegExp( wild );
     try {
-      boost::regex rx( reg );
+      regex rx( reg );
       if ( isFile( name ) ){
 	// it is just 1 file
 	string::size_type pos = name.rfind( "/" );
@@ -193,32 +198,27 @@ namespace TiCC {
 	else {
 	  fname = name;
 	}
-	if ( boost::regex_search( fname, rx ) )
+	if ( regex_search( fname, rx ) )
 	  result.push_back( name );
 	return result;
       }
       else if ( !isDir( name ) ){
-	cerr << "the name '" << name
-	     << "' doesn't match a file or directory." << endl;
-	exit(EXIT_FAILURE);
+	string mess = "TiCC::searchFilesMatch: the name '" + name
+	  + "' doesn't match a file or directory.";
+	throw runtime_error( mess );
       }
       gatherFilesMatch( name, rx, result, recurse );
     }
-    catch( boost::regex_error& e ){
-      cerr << "invalid regexp: " << e.what() << endl;
-      exit(EXIT_FAILURE);
+    catch( regex_error& e ){
+      string mess = "TiCC::searchFilesMatch: invalid regexp: ";
+      mess += e.what();
+      throw runtime_error( mess );
+    }
+    catch( ... ){
+      throw;
     }
     return result;
   }
-#else
-  vector<string> searchFilesMatch( const string& name,
-				   const string& match,
-				   bool recurse ){
-    cerr << "REGEXP support not available" << endl;
-    cerr << "  attempting lame extension matching instead" << endl;
-    return searchFilesExt( name, match, recurse );
-  }
-#endif
 
   bool createTruePath( const string& path ){
     // attempt to open a path /a/b/c/ from an expression like:
@@ -226,21 +226,20 @@ namespace TiCC {
     // or ./a/b/c/ from expressions like
     // a/b/c/
 
-    ofstream os1( path.c_str() );
+    ofstream os1( path );
     if ( !os1.good() ){
       // it fails
       // attempt to create the path
-      vector<string> parts;
-      int num = split_at( path, parts, "/" );
-      if ( num > 0 ){
+      vector<string> parts = split_at( path, "/" );
+      if ( parts.size() > 0 ){
 	//  at least one /
 	string newpath;
 	if ( path[0] == '/' )
 	  newpath = "/";
 	else
 	  newpath = "./";
-	for ( size_t i=0; i < parts.size(); ++i ){
-	  newpath += parts[i] + "/";
+	for ( auto const& p : parts ){
+	  newpath += p + "/";
 	  //	  cerr << "mkdir path = " << newpath << endl;
 	  int status = mkdir( newpath.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH );
 	  if ( status != 0 && errno != EEXIST ){
@@ -264,7 +263,7 @@ namespace TiCC {
       if ( !createTruePath( path ) )
 	return false;
     }
-    ofstream os( name.c_str() );
+    ofstream os( name );
     if ( !os.good() ){
       return false;
     }
