@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -24,6 +24,8 @@
       lamasoftware (at ) science.ru.nl
 */
 
+#include "ticcutils/SocketBasics.h"
+
 #include <string>
 #include <stdexcept>
 #include <iostream>
@@ -37,7 +39,6 @@
 
 #include "config.h"
 #include "ticcutils/StringOps.h"
-#include "ticcutils/SocketBasics.h"
 
 using namespace std;
 
@@ -58,10 +59,14 @@ namespace Sockets {
   }
 
   // #define KEEP // experiment with keep-alive
+  //#define DEBUG
 
   bool Socket::read( string& line ) {
     if ( !isValid() ){
       mess = "read: socket invalid";
+#ifdef DEBUG
+      cerr << mess << endl;
+#endif
       return false;
     }
     line = "";
@@ -77,7 +82,12 @@ namespace Sockets {
     while ( last_read != 10 ) { // read 1 character at a time upto \n
       bytes_read = ::read( sock, &last_read, 1 );
       if ( bytes_read <= 0) {
+#ifdef DEBUG
+	cerr << "read res = " << bytes_read  << " ( " << strerror(bytes_read) << ")" << endl;
+#endif
 	// The other side may have closed unexpectedly
+	::close(sock);
+	sock = -1;
 	break;
       }
       if ( ( last_read != 10 ) && ( last_read !=13 ) ) {
@@ -87,6 +97,10 @@ namespace Sockets {
     }
     if ( bytes_read < 0 ) {
       mess = string("connection closed ") + strerror( bytes_read );
+#ifdef DEBUG
+      cerr << mess << endl;
+#endif
+
       return false;
     }
     else if ( bytes_read == 0 ) {
@@ -97,8 +111,6 @@ namespace Sockets {
     }
   }
 
-  //#define DEBUG
-
   bool Socket::read( string& result, unsigned int timeout ) {
     // a getline for nonblocking connections.
     // retry for a few special cases until timeout reached.
@@ -139,6 +151,8 @@ namespace Sockets {
 	}
 	else {
 	  mess = strerror( res );
+	  ::close(sock);
+	  sock = -1;
 	  return false;
 	}
       }
@@ -165,14 +179,17 @@ namespace Sockets {
 	  cerr << "write res = " << this_write  << " ( " << strerror(this_write) << ")" << endl;
 #endif
 	} while ( (this_write < 0) && (errno == EINTR) );
-	if (this_write <= 0)
+	if (this_write <= 0){
 	  break;
+	}
 	bytes_sent += this_write;
 	str += this_write;
       }
       if ( bytes_sent < count ) {
 	mess = "write: failed to sent " + TiCC::toString(count - bytes_sent) +
 	  " bytes out of " + TiCC::toString(count);
+	::close(sock);
+	sock = -1;
 	return false;
       }
     }
@@ -203,12 +220,16 @@ namespace Sockets {
 	}
 	else {
 	  mess = strerror( res );
+	  ::close(sock);
+	  sock = -1;
 	  return false;
 	}
       }
       if ( bytes_sent < count ) {
 	mess = "write: failed to sent " + TiCC::toString(count - bytes_sent) +
 	  " bytes out of " + TiCC::toString(count);
+	::close(sock);
+	sock = -1;
 	return false;
       }
     }
@@ -221,9 +242,10 @@ namespace Sockets {
       m = "socket " + TiCC::toString(sock);
     else
       m = "invalid socket ";
-    if ( !mess.empty() )
+    if ( !mess.empty() ){
       m += ": " + mess;
-    return mess;
+    }
+    return m;
   }
 
   bool Socket::setBlocking( ) {
@@ -308,14 +330,14 @@ namespace Sockets {
     }
     else {
       aip = res;
-      while( aip ){
+      while ( aip ){
 	sock = socket( aip->ai_family, aip->ai_socktype, aip->ai_protocol);
 	if ( sock > 0 )
 	  break;
 	sock = -1;
 	aip = aip->ai_next;
       }
-      if ( sock < 0 ){
+      if ( sock < 0 || aip == 0 ){
 	mess = string( "ClientSocket: Socket could not be created: (" )
 	  + strerror(errno) + ")";
       }
