@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -25,21 +25,22 @@
 
 */
 
+#include "ticcutils/StringOps.h"
+
+#include <cerrno>
+#include <cfloat>
+#include <cstdlib>
 #include <algorithm>
 #include <string>
 #include <iostream>
 #include <sstream>
-
-#include <cerrno>
-#include <cfloat>
 #include "ticcutils/Version.h"
-#include "ticcutils/StringOps.h"
 
 using namespace std;
 namespace TiCC {
 
   string BuildInfo() {
-    // cannot be defined in the header becaus otherwise __DATE__ ans
+    // cannot be defined in the header because otherwise __DATE__ and
     // __TIME__ would be dynamic. (changing every time it is included)
     return VersionName() + "-" + Version() + ". Compiled on "
       + __DATE__ + " " + __TIME__;
@@ -127,6 +128,9 @@ namespace TiCC {
     // split a string into substrings, using seps as separator
     // silently skip empty entries (e.g. when two or more separators co-incide)
     // unless exact=true;
+    if ( sep.empty() ){
+      throw runtime_error( "TiCC::split_at(): separator is empty!" );
+    }
     results.clear();
     string::size_type pos = 0;
     while ( pos != string::npos ){
@@ -146,12 +150,53 @@ namespace TiCC {
     return results.size();
   }
 
+  vector<string> split_at( const string& src,
+			   const string& sep,
+			   size_t max ){
+    // split a string into substrings, using seps as separator
+    // silently skip empty entries (e.g. when two or more separators co-incide)
+    // if max > 0, limit the size off the result to max,
+    // leaving the remainder in the last part of the result
+    if ( sep.empty() ){
+      throw runtime_error( "TiCC::split_at(): separator is empty!" );
+    }
+    vector<string> results;
+    size_t cnt = 0;
+    string::size_type pos = 0;
+    while ( pos != string::npos ){
+      string res;
+      string::size_type p = src.find( sep, pos );
+      if ( p == string::npos ){
+	res = src.substr( pos );
+	pos = p;
+      }
+      else {
+	res = src.substr( pos, p - pos );
+	pos = p + sep.length();
+      }
+      if ( !res.empty() ){
+	++cnt;
+	results.push_back( res );
+      }
+      if ( max != 0 && cnt >= max-1 ){
+	if ( pos != string::npos ){
+	  results.push_back( src.substr( pos ) );
+	}
+	break;
+      }
+    }
+    return results;
+  }
+
   size_t split_at_first_of( const string& src, vector<string>& results,
 			    const string& seps, bool exact ){
     // split a string into substrings, using the characters in seps
     // as seperators
     // silently skip empty entries (e.g. when two or more seperators co-incide)
     // unless exact=true;
+    if ( seps.empty() ){
+      throw runtime_error( "TiCC::split_at_first_of(): separators are empty!" );
+    }
     results.clear();
     string::size_type s = 0;
     while ( s != string::npos ){
@@ -172,35 +217,95 @@ namespace TiCC {
     return results.size();
   }
 
+  vector<string> split_at_first_of( const string& src,
+				    const string& seps,
+				    size_t max ){
+    // split a string into substrings, using the characters in seps
+    // as seperators
+    // silently skip empty entries (e.g. when two or more seperators co-incide)
+    // if max > 0, limit the size of the result to max,
+    // leaving the remainder in the last part of the result
+    if ( seps.empty() ){
+      throw runtime_error( "TiCC::split_at_first_of(): separators are empty!" );
+    }
+    vector<string> results;
+    size_t cnt = 0;
+    string::size_type pos = 0;
+    while ( pos != string::npos ){
+      string res;
+      string::size_type e = src.find_first_of( seps, pos );
+      if ( e == string::npos ){
+	res = src.substr( pos );
+	pos = e;
+      }
+      else {
+	res = src.substr( pos, e - pos );
+	pos = e+1;
+      }
+      if ( !res.empty() ){
+	results.push_back( res );
+	++cnt;
+      }
+      if ( max != 0 && cnt >= max-1 ){
+	if ( pos != string::npos ){
+	  results.push_back( src.substr( pos ) );
+	}
+	break;
+      }
+    }
+    return results;
+  }
+
   // format weird strings (like UTF8, LATIN1) printable
   // useful for debugging
   string format_nonascii( const string& s ){
     stringstream os;
     os << showbase << hex;
-    unsigned int i;
-    for ( i=0; i < s.length(); ++i )
-      if ( isprint(s[i]) && (int)s[i] > 31 )
-	os << s[i];
-      else
-	os << "-" << (short int)s[i] << "-";
+    for ( const auto& c : s ){
+      if ( isprint(c) && (int)c > 31 ){
+	os << c;
+      }
+      else {
+	os << "-" << (short int)c << "-";
+      }
+    }
     os << noshowbase << dec;
     return os.str();
   }
 
-  std::string basename( const std::string& path ){
+  string basename( const string& path ){
     string::size_type pos = path.rfind( "/" );
-    if ( pos != string::npos )
+    if ( pos != string::npos ){
       return path.substr(pos+1);
-    else
+    }
+    else {
       return path;
+    }
   }
 
-  std::string dirname( const std::string& path ){
+  string dirname( const string& path ){
     string::size_type pos = path.rfind( "/" );
-    if ( pos != string::npos )
+    if ( pos != string::npos ){
       return path.substr(0,pos);
-    else
+    }
+    else {
       return ".";
+    }
+  }
+
+  string realpath( const string& path ){
+    string result;
+    if ( path.empty() ){
+      return result;
+    }
+    const char *in = path.c_str();
+    char *out = 0;
+    char *res = ::realpath( in, out );
+    if ( res ){
+      result = string(res);
+      free( res);
+    }
+    return result;
   }
 
 } // namespace TiCC
