@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -24,68 +24,30 @@
       lamasoftware (at ) science.ru.nl
 */
 
+#include "config.h"
+#include "ticcutils/Tar.h"
+
 #include <cstring>
 #include <cerrno>
 #include <string>
 #include <iostream>
 #include <fstream>
 #include <unistd.h>
-#include "config.h"
 #include <sys/fcntl.h>
 #include "ticcutils/StringOps.h"
-#ifdef HAVE_BOOST_REGEX
-#include <boost/regex.hpp>
+#if HAVE_BOOST_REGEX
+#  include <boost/regex.hpp>
+#  define regex boost::regex
+#  define regex_error boost::regex_error
+#  define regex_search boost::regex_search
+#else
+#  include <regex>
 #endif
-#include "ticcutils/Tar.h"
 
 using namespace std;
 
 namespace TiCC {
 
-#ifndef HAVE_LIBTAR_H
-
-  void tarfail( const string& s ){
-    throw runtime_error( "unable to execute '" + s + " 'tar support not enabled" );
-  }
-
-  tar::tar() {
-  }
-
-  tar::~tar(){
-  }
-
-  bool tar::open( const string& ){
-    tarfail( "open()" );
-    return false;
-  }
-
-  bool tar::close() {
-    return false;
-  }
-
-  bool tar::extract_file_names( vector<string>&,
-				const string& ){
-    tarfail( "extract_file_names()" );
-    return false;
-  }
-
-  bool tar::extract_file_names_match( vector<string>&,
-				      const string& ){
-    tarfail( "extract_file_names_match()" );
-    return false;
-  }
-
-  bool tar::extract_ifstream( const string&, ifstream& ){
-    tarfail( "extract_ifstream()" );
-    return false;
-  }
-
-  bool tar::next_ifstream( ifstream&, string&){
-    tarfail( "next_ifstream()" );
-    return false;
-  }
-
-#else
   tar::tar() : tar_file(0)
   {}
 
@@ -139,12 +101,11 @@ namespace TiCC {
     return true;
   }
 
-#ifdef HAVE_BOOST_REGEX
   static string wildToRegExp( const string& wild ){
     // convert 'shell'-like wildcards into a regexp
     string result;
-    for ( size_t i=0; i < wild.length(); ++i ){
-      switch( wild[i] ){
+    for ( auto const& c : wild ){
+      switch( c ){
       case '*':
 	result += ".*";
 	break;
@@ -152,11 +113,10 @@ namespace TiCC {
 	result += ".";
 	break;
       case '.':
-	result += "\\";
-	result += wild[i];
+	result += "\\.";
 	break;
       default:
-	result += wild[i];
+	result += c;
       }
     }
     //    cerr << "wild to regexp: " << wild << " ==> " << result << endl;
@@ -179,12 +139,12 @@ namespace TiCC {
     }
     string pat = wildToRegExp( wild );
     try {
-      boost::regex rx( pat );
+      regex rx( pat );
       stat = th_read( local_tar );
       while ( stat == 0  ) {
 	if ( TH_ISREG( local_tar ) ){
 	  string name = local_tar->th_buf.name;
-	  if ( boost::regex_search( name, rx ) ){
+	  if ( regex_search( name, rx ) ){
 	    result.push_back( name );
 	  }
 	  tar_skip_regfile( local_tar );
@@ -193,20 +153,13 @@ namespace TiCC {
       }
       tar_close( local_tar );
     }
-    catch( boost::regex_error& e ){
-      cerr << "invalid regexp: " << e.what() << endl;
-      exit(EXIT_FAILURE);
+    catch( regex_error& e ){
+      string mess = "tar:extract_file_names_match() invalid regexp: ";
+      mess += e.what();
+      throw runtime_error( mess );
     }
     return true;
   }
-#else
-  bool tar::extract_file_names_match( vector<string>& result,
-				      const string& pat ){
-    cerr << "tar::extract() REGEXP support not available" << endl;
-    cerr << "  attempting lame extension matching instead" << endl;
-    return extract_file_names( result, pat );
-  }
-#endif
 
   bool tar::extract_ifstream( const string& name, ifstream& result ){
     result.close();
@@ -232,12 +185,14 @@ namespace TiCC {
 	    cerr << "tar_extract_regfile(): " << strerror(errno) << endl;
 	    return false;
 	  }
-	  result.open(tmpfile.c_str() );
+	  result.open( tmpfile );
 	  if ( result.good() ){
+	    tar_close( local_tar );
 	    return true;
 	  }
 	  else {
 	    cerr << "problem with " << tmpfile << endl;
+	    tar_close( local_tar );
 	    return false;
 	  }
 	}
@@ -280,7 +235,7 @@ namespace TiCC {
 	  cerr << "tar_extract_regfile(): " << strerror(errno) << endl;
 	  return false;
 	}
-	result.open(tmpfile.c_str() );
+	result.open( tmpfile );
 	if ( result.good() ){
 	  name = tar_file->th_buf.name;
 	  return true;
@@ -305,6 +260,5 @@ namespace TiCC {
     }
     return true;
   }
-#endif
 
 }
