@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -24,12 +24,12 @@
       lamasoftware (at ) science.ru.nl
 */
 
+#include "ticcutils/TreeHash.h"
+
 #include <iostream>
 #include <cstdlib>
 #include <cstring>
 
-#include "ticcutils/TreeHash.h"
-
 using namespace std;
 
 namespace Hash {
@@ -56,19 +56,16 @@ namespace Hash {
 
   unsigned int StringHash::Hash( const string& name ){
     unsigned int idx = 0;
-#pragma omp critical(tree_mod)
-    {
-      HashInfo *info = StringTree.Retrieve( name );
-      if ( !info ){
-	info = new HashInfo( name, ++NumOfTokens );
-	info = (HashInfo *)StringTree.Store( name, info );
-      }
-      idx = info->Index();
-      if ( idx >= rev_index.size() ){
-	rev_index.resize( rev_index.size() + 1000 );
-      }
-      rev_index[idx] = info;
+    HashInfo *info = StringTree.Retrieve( name );
+    if ( !info ){
+      info = new HashInfo( name, ++NumOfTokens );
+      info = reinterpret_cast<HashInfo *>(StringTree.Store( name, info ));
     }
+    idx = info->Index();
+    if ( idx >= rev_index.size() ){
+      rev_index.resize( rev_index.size() + 1000 );
+    }
+    rev_index[idx] = info;
     return idx;
   }
 
@@ -102,18 +99,15 @@ namespace Hash {
   Lexicon::~Lexicon(){}
 
   LexInfo *Lexicon::Lookup( const string& name ) const {
-    return (LexInfo *)LexTree.Retrieve( name );
+    return reinterpret_cast<LexInfo *>(LexTree.Retrieve( name ));
   }
 
   LexInfo *Lexicon::Store( const string& name, const string& translation ){
     LexInfo *info = 0;
-#pragma omp critical(tree_mod)
-    {
-      info = LexTree.Retrieve( name );
-      if ( !info ){
-	info = new LexInfo( name, translation );
-	info = LexTree.Store( name, info );
-      }
+    info = LexTree.Retrieve( name );
+    if ( !info ){
+      info = new LexInfo( name, translation );
+      info = LexTree.Store( name, info );
     }
     return info;
   }
