@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2016
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -32,6 +32,7 @@
 #include <stdexcept>
 
 #include "ticcutils/StringOps.h"
+#include "ticcutils/TreeHash.h"
 #include "ticcutils/PrettyPrint.h"
 #include "ticcutils/zipper.h"
 #include "ticcutils/Tar.h"
@@ -41,14 +42,21 @@
 #include "ticcutils/CommandLine.h"
 #include "ticcutils/Configuration.h"
 #include "ticcutils/LogStream.h"
+#include "ticcutils/Unicode.h"
 
 using namespace std;
 using namespace TiCC;
+using namespace icu;
 
 void helper(){
   throw runtime_error("fout");
 }
 
+int helper2(){
+  throw runtime_error("mis");
+  return 8;
+}
+
 void test_throw(){
   assertThrow( helper(), runtime_error );
   assertNoThrow( 4==7 );
@@ -61,16 +69,16 @@ void test_nothrow(){
 
 void test_opts_basic(){
   startTestSerie( "we testen basic commandline opties." );
+  bool opt_dbg=false;
   CL_Options opts1;
+  opts1.set_debug(opt_dbg);
   opts1.allow_args( "t:fh" );
   // -t mist een optie
   assertThrow( opts1.parse_args( "-t -f -h" ), OptionError );
   // onbekende optie
   assertThrow( opts1.init( "-a" ), OptionError );
-  // -f heeft onterecht een parameter
-  assertThrow( opts1.parse_args( "-t1 -f bla -h"), OptionError );
-  // parameters aan einde ==> massopts.
-  assertNoThrow( opts1.parse_args( "-t1 -f -h bla") );
+  // -f heeft onterecht een parameter ==> massopts
+  assertNoThrow( opts1.parse_args( "-t1 -f bla -h") );
   CL_Options opts2;
   opts2.allow_args( "", "true:,false" );
   // --true mist een optie
@@ -154,7 +162,7 @@ void test_opts_basic(){
   assertThrow( opts9.extract('r', myint ), OptionError );
   CL_Options opts10;
   opts10.allow_args( "", "test::,qed,data:" );
-  //  opts10.set_debug(true);
+  //  opts10.set_debug(opt_dbg);
   // --test heeft optionele optie. qed is een stoorzender
   assertNoThrow( opts10.parse_args( "--test 1 --test=2 --qed --test --test=3 --data=5.6 --data=appel")  );
   ts.clear();
@@ -199,11 +207,47 @@ void test_opts_basic(){
   auto v = opts13.getMassOpts();
   assertEqual( v.size(), 2 );
   CL_Options opts14;
-  opts14.set_debug(true);
+  opts14.set_debug(opt_dbg);
   opts14.parse_args( "-a b -a c oke -d\"-fiets --appel peer \" --fout=goed toch" );
   assertEqual( opts14.toString(), "-ab -ac -d-fiets --appel peer  --fout=goed" );
   v = opts14.getMassOpts();
   assertEqual( v.size(), 2 );
+  CL_Options opts15;
+  opts15.set_debug(opt_dbg);
+  opts15.parse_args( "--fout=goed\\mis --jan=gek" );
+  assertEqual( opts15.toString(), "--fout=goed\\mis --jan=gek" );
+  string res;
+  opts15.extract("fout", res );
+  assertEqual( res, "goed\\mis" );
+  opts15.extract("jan", res );
+  assertEqual( res, "gek" );
+  CL_Options opts16;
+  opts16.allow_args( "", "test:" );
+  opts16.set_debug(opt_dbg);
+  opts16.parse_args( "--test goed --test=prima --test niet=eens --test=wel=eens" );
+  opts16.extract("test", res );
+  assertEqual( res, "goed" );
+  opts16.extract("test", res );
+  assertEqual( res, "prima" );
+  opts16.extract("test", res );
+  assertEqual( res, "niet=eens" );
+  opts16.extract("test", res );
+  assertEqual( res, "wel=eens" );
+  CL_Options opts17;
+  // new feature: check for stray mass opts inside commandline
+  opts17.allow_args( "ab:c", "aap" );
+  opts17.set_debug(true);
+  opts17.parse_args( "-a file1 -b prima de luxe --aap file2 -c file3 file4" );
+  opts17.extract("a", res );
+  assertEqual( res, "" );
+  opts17.extract("b", res );
+  assertEqual( res, "prima" );
+  opts17.extract("aap", res );
+  assertEqual( res, "" );
+  opts17.extract("c", res );
+  assertEqual( res, "" );
+  vector<string> mo2 = opts17.getMassOpts();
+  assertEqual( mo2.size(), 6  );
 }
 
 void test_opts( CL_Options& opts ){
@@ -234,6 +278,7 @@ void test_opts( CL_Options& opts ){
 void test_subtests_fail(){
   startTestSerie( "we testen subtests, met faal." );
   assertThrow( helper(), range_error );
+  assertEqual( helper2(), 4 );
   assertEqual( (1 + 2), (2 + 1) );
   assertEqual( 4, 5 );
 }
@@ -294,16 +339,36 @@ void test_split(){
   string line = "De kat krabt de krullen\n van de   trap.";
   vector<string> res;
   int cnt = split( line, res );
-  //  cerr << "after split: " << res << endl;
   assertEqual( cnt, 8 );
   assertEqual( res[5], "van" );
+  vector<string> res2 = split( line );
+  assertEqual( res2.size(), 8 );
+  assertEqual( res2[6], "de" );
+  vector<string> res3 = split( line, 3 );
+  assertEqual( res3.size(), 3 );
+  assertEqual( res3[1], "kat" );
+  assertEqual( res3[2], "krabt de krullen\n van de   trap." );
+  vector<string> res4 = split( line, 24 );
+  assertEqual( res4.size(), 8 );
+  assertEqual( res4[1], "kat" );
+  assertEqual( res4[2], "krabt" );
+  vector<string> res5 = split( line, -1 );
+  assertEqual( res5.size(), 8 );
+  assertEqual( res5[1], "kat" );
+  assertEqual( res5[2], "krabt" );
+  vector<string> res6 = split( line, 0 );
+  assertEqual( res6.size(), 8 );
+  assertEqual( res6[1], "kat" );
+  assertEqual( res6[2], "krabt" );
+  vector<string> res7 = split( string("APPELTAART"), 2 );
+  assertEqual( res7.size(), 1 );
+  assertEqual( res7[0], "APPELTAART" );
 }
 
 void test_split_exact(){
   string line = "1 2  4    8  10";
   vector<string> res;
   int cnt = split( line, res, true );
-  //  cerr << "after split: " << res << endl;
   assertEqual( cnt, 10 );
   assertEqual( res[5], "" );
 }
@@ -312,7 +377,6 @@ void test_split_at_exact(){
   string line = "1/2//4////8//10";
   vector<string> res;
   int cnt = split_at( line, res, "/", true );
-  //  cerr << "after split: " << res << endl;
   assertEqual( cnt, 10 );
   assertEqual( res[5], "" );
 }
@@ -322,8 +386,14 @@ void test_split_at(){
   vector<string> res;
   int cnt = split_at( line, res, "rare" );
   assertEqual( cnt, 8 );
-  //  cerr << "after split: " << res << endl;
   assertEqual( res[5], "van" );
+  vector<string> res2 = split_at( line, "rare" );
+  assertEqual( res2.size(), 8 );
+  assertEqual( res2[6], "de" );
+  vector<string> res3 = split_at( line, "rare", 4 );
+  assertEqual( res3.size(), 4 );
+  assertEqual( res3[2], "krabt" );
+  assertEqual( res3[3], "derarekrullen\nrarevanrarederaretrap." );
 }
 
 void test_split_at_first(){
@@ -331,8 +401,14 @@ void test_split_at_first(){
   vector<string> res;
   int cnt = split_at_first_of( line, res, ".,?!:;" );
   assertEqual( cnt, 8 );
-  //  cerr << "after split: " << res << endl;
   assertEqual( res[5], "van" );
+  vector<string> res2 = split_at_first_of( line, ".,?!:;" );
+  assertEqual( res2.size(), 8 );
+  assertEqual( res2[6], "de" );
+  vector<string> res3 = split_at_first_of( line, ".,?!:;", 7 );
+  assertEqual( res3.size(), 7 );
+  assertEqual( res3[4], "krullen" );
+  assertEqual( res3[6], ",;.;de!trap." );
 }
 
 void test_split_at_first_exact(){
@@ -369,17 +445,46 @@ void test_lowercase(){
   assertEqual( res, "een camelcapped zin." );
 }
 
+void test_lexicon(){
+  Hash::Lexicon lex;
+  lex.Store( "appel", "apple" );
+  lex.Store( "peer", "pear" );
+  lex.Store( "appeltaart", "applepie" );
+  Hash::LexInfo *info = lex.Lookup( "cake" );
+  assertEqual( (void*)info, (void*)0 );
+  info = lex.Lookup( "appel" );
+  assertEqual( info->Trans(), "apple" );
+}
+
+void test_treehash(){
+  Hash::StringHash sh;
+  size_t index = sh.Hash( "appel" );
+  assertEqual( index, 1 );
+  index = sh.Hash( "peer" );
+  assertEqual( index, 2 );
+  index = sh.Hash( "appeltaart" );
+  assertEqual( index, 3 );
+  index = sh.Hash( "peer" );
+  assertEqual( index, 2 );
+  assertEqual( sh.NumOfEntries(), 3 );
+  assertEqual( sh.ReverseLookup( 3 ), "appeltaart" );
+}
+
 void test_base_dir(){
-  assertEqual( basename("/foo/bar" ), "bar" );
-  assertEqual( dirname("/foo/bar" ), "/foo" );
-  assertEqual( basename("foo/bar" ), "bar" );
-  assertEqual( dirname("foo/bar" ), "foo" );
-  assertEqual( basename("foobar" ), "foobar" );
-  assertEqual( dirname("foobar" ), "." );
-  assertEqual( basename("/" ), "/" );
-  assertEqual( dirname("/" ), "/" );
-  assertEqual( basename("." ), "." );
-  assertEqual( dirname("." ), "." );
+  assertEqual( TiCC::basename("/foo/bar" ), "bar" );
+  assertEqual( TiCC::dirname("/foo/bar" ), "/foo" );
+  assertEqual( TiCC::basename("foo/bar" ), "bar" );
+  assertEqual( TiCC::dirname("foo/bar" ), "foo" );
+  assertEqual( TiCC::basename("foobar" ), "foobar" );
+  assertEqual( TiCC::dirname("foobar" ), "." );
+  assertEqual( TiCC::basename("/" ), "" );
+  assertEqual( TiCC::dirname("/" ), "" );
+  assertEqual( TiCC::basename("." ), "." );
+  assertEqual( TiCC::dirname("." ), "." );
+}
+
+void test_realpath(){
+  assertEqual( realpath( "" ), "" );
 }
 
 void test_bz2compression( const string& path ){
@@ -417,6 +522,7 @@ void test_tar( const string& path ){
   assertTrue( mytar.next_ifstream( tmp, name ) );
   assertEqual( name, "small.txt" );
   assertTrue( mytar.next_ifstream( tmp, name ) );
+  assertEqual( name, "sub.txt" );
   assertTrue( mytar.next_ifstream( tmp, name ) );
   assertEqual( name, "sub1/sub.txt" );
   assertNoThrow( mytar.extract_ifstream( "sub1/sub.txt", tmp ) );
@@ -425,13 +531,8 @@ void test_tar( const string& path ){
   assertEqual( line, "a testfile." );
   assertNoThrow( mytar.extract_file_names( res, ".xml" ) );
   assertEqual( res.size(), 1 );
-#ifdef HAVE_BOOST_REGEX
   assertNoThrow( mytar.extract_file_names_match( res, "s*b" ) );
   assertEqual( res.size(), 3 );
-#else
-  assertNoThrow( mytar.extract_file_names_match( res, "txt" ) );
-  assertEqual( res.size(), 3 );
-#endif
 }
 
 void test_fileutils( const string& path ){
@@ -452,7 +553,6 @@ void test_fileutils( const string& path ){
   assertTrue( createPath( "/tmp/test/silly/files/path/raar" ) );
   assertFalse( createPath( "/tmp/test/silly/files/path/raar/sub" ) );
 
-#ifdef HAVE_BOOST_REGEX
   assertNoThrow( res = searchFilesMatch( path, "*.txt", false ) );
   // non recursive. should match small.txt
   assertEqual( res.size(), 1 );
@@ -465,7 +565,6 @@ void test_fileutils( const string& path ){
   assertNoThrow( res = searchFilesMatch( path, "s*[lb].txt" ) );
   // should match small.txt and sub1/sub.txt
   assertEqual( res.size(), 2 );
-#endif
 }
 
 void test_configuration( const string& path ){
@@ -486,6 +585,8 @@ void test_configuration( const string& path ){
   assertEqual( oud, "gek" );
   c.setatt( "pief", "paf", "poef" );
   c.clearatt( "piet", "test" );
+  att = c.lookUp( "Truus", "test" );
+  assertEqual( att, "met een \n er in en een \r!" );
   assertNoThrow( c.create_configfile( "/tmp/test.cfg" ) );
   Configuration c2;
   assertTrue( c2.fill( "/tmp/test.cfg" ) );
@@ -499,6 +600,28 @@ void test_configuration( const string& path ){
   assertEqual( att, "paf" );
   att = c2.lookUp( "piet", "test" );
   assertEqual( att, "" );
+  att = c.lookUp( "Truus", "test" );
+  assertEqual( att, "met een \n er in en een \r!" );
+  Configuration c3;
+  assertTrue( c3.fill( path + "testconfig.cfg" ) );
+  att = c3.lookUp( "jan" );
+  assertEqual( att, "gek" );
+  att = c3.lookUp( "pief", "poef" );
+  assertEqual( att, "" );
+  att = c3.lookUp( "piet", "test" );
+  assertEqual( att, "ook gek" );
+  c3.merge( c2 ); //default is NOT to override what is already there
+  att = c3.lookUp( "jan" );
+  assertEqual( att, "gek" ); // not overridden ?
+  att = c3.lookUp( "pief", "poef" );
+  assertEqual( att, "paf" );
+  att = c3.lookUp( "piet", "test" ); // empty in c2, not changed in c3
+  assertEqual( att, "ook gek" );
+  c3.merge( c2, true ); // Now override
+  att = c3.lookUp( "jan" );
+  assertEqual( att, "normaal" ); // is overridden now?
+  att = c3.lookUp( "piet", "test" ); // empty in c2, still not changed in c3
+  assertEqual( att, "ook gek" );
 }
 
 void test_logstream( const string& path ){
@@ -533,6 +656,133 @@ void test_logstream( const string& path ){
   assertEqual( system( cmd.c_str() ), 0 );
 }
 
+void test_unicode( const string& path ){
+  UnicodeString u1 = L'私';
+  UChar32 uc1 = U'\U00007981';
+  UChar32 uc2 = U'\U00007982';
+  UnicodeString u2 = uc1;
+  u2 += UnicodeString( uc2 );
+  string s1 = UnicodeToUTF8( u1 );
+  assertEqual( s1 , "私" );
+  string s2 = UnicodeToUTF8( u2 );
+  assertEqual( s2 , "禁禂" );
+  ifstream in( path + "utf16bom.nl" );
+  string line;
+  getline( in, line );
+  assertFalse( line == "Hier staat een BOM voor. æ en ™ om te testen." );
+  UnicodeString u3 = UnicodeFromEnc( line, "UTF16" );
+  string s3 = UnicodeToUTF8(  u3 );
+  assertEqual( s3, "Hier staat een BOM voor. æ en ™ om te testen." );
+  UnicodeString greek1 = "ἀντιϰειμένου";
+  UnicodeString greek2 = "ἀντιϰειμένου";
+  assertFalse( greek1 == greek2 ); // different normalizations!
+  UnicodeNormalizer N;
+  UnicodeString ng1 = N.normalize( greek1 );
+  UnicodeString ng2 = N.normalize( greek2 );
+  assertEqual( UnicodeToUTF8(ng1), UnicodeToUTF8(ng2) );
+  N.setMode("NFD");
+  UnicodeString ng11 = N.normalize( greek1 );
+  UnicodeString ng12 = N.normalize( greek2 );
+  assertEqual( UnicodeToUTF8(ng11), UnicodeToUTF8(ng12) );
+  string utf8_1 = "ἀντιϰειμένου";
+  string utf8_2 = "ἀντικειμένου";
+  assertEqual( TiCC::utf8_uppercase( utf8_1 ), "ἈΝΤΙΚΕΙΜΈΝΟΥ" );
+  assertEqual( TiCC::utf8_lowercase( "ἈΝΤΙΚΕΙΜΈΝΟΥ" ), utf8_2 );
+  assertEqual( TiCC::utf8_uppercase( "æ en ß en œ" ), "Æ EN SS EN Œ" );
+}
+
+void test_unicode_split(){
+  string line8 = "De kat krabt de krullen\n van de   trap.";
+  UnicodeString line = TiCC::UnicodeFromUTF8( line8 );
+  vector<UnicodeString> res = split_at( line, "de" );
+  assertEqual( res.size(), 3 );
+  assertEqual( TiCC::UnicodeToUTF8(res[0]), "De kat krabt " );
+  assertEqual( TiCC::UnicodeToUTF8(res[1]), " krullen\n van " );
+  assertEqual( TiCC::UnicodeToUTF8(res[2]), "   trap." );
+  res = split( line, 3 );
+  assertEqual( res.size(), 3 );
+  assertEqual( res[1], "kat" );
+  assertEqual( res[2], "krabt de krullen\n van de   trap." );
+  res = split( line, 24 );
+  assertEqual( res.size(), 8 );
+  assertEqual( res[1], "kat" );
+  assertEqual( res[2], "krabt" );
+  assertEqual( res[4], "krullen" );
+  assertEqual( res[5], "van" );
+  UnicodeString vies = "em—dash, en–dash, bar―, bar―――, 3em⸻dash, FullWidth－HyphenMinus,";
+  res = split_at( vies, "," );
+  assertEqual( res.size(), 6 );
+  assertEqual( res[5], " FullWidth－HyphenMinus" );
+  UnicodeString seps =  "—–―⸻－";
+  res = split_at_first_of( vies, seps );
+  assertEqual( res.size(), 7 );
+  assertEqual( res[0], "em" );
+  assertEqual( res[2], "dash, bar" );
+  assertEqual( res[4], ", 3em" );
+  assertEqual( res[6], "HyphenMinus," );
+}
+
+void test_unicode_regex( ){
+  string pattern1 = "^(\\p{Lu}{1,2}\\.{1,2}(\\p{Lu}{1,2}\\.{1,2})*)(\\p{Lu}{0,2})$";
+  UnicodeRegexMatcher test1( UnicodeFromUTF8(pattern1), "test1" );
+  UnicodeString pre, post;
+  UnicodeString us = "A.N.W.B.";
+  assertTrue( test1.match_all( us, pre, post ) );
+  us = "A.N.W..B";
+  assertTrue( test1.match_all( us, pre, post ) );
+  us = "A.NON.W.B.";
+  assertFalse( test1.match_all( us, pre, post ) );
+  string pattern2 = "(?:de|het|een)_(\\p{Lu}+)(?:-(?:\\p{L}*)|\\Z)";
+  UnicodeRegexMatcher test2( UnicodeFromUTF8(pattern2), "test2" );
+  us = "een_CDA-minister";
+  assertTrue( test2.match_all( us, pre, post ) );
+  string result = TiCC::UnicodeToUTF8( test2.get_match( 0 ) );
+  assertEqual( result, "CDA" );
+  us = "de_VVD";
+  assertTrue( test2.match_all( us, pre, post ) );
+  result = TiCC::UnicodeToUTF8( test2.get_match( 0 ) );
+  assertEqual( result, "VVD" );
+}
+
+void test_unicode_filters( const string& path ){
+  UniFilter filt;
+  assertNoThrow( filt.init( "‘ > \\' ; ’ > \\' ;  \\` > \\' ; ´ > \\' ;", "quote_filter" ) );
+  UnicodeString vies = "`vies´ en ‘smerig’ en `apart´";
+  UnicodeString schoon = filt.filter( vies );
+  assertEqual( schoon, "\'vies\' en \'smerig\' en \'apart\'" );
+  UniFilter filt2;
+  assertNoThrow( filt2.fill( path + "quotes.filter") );
+  schoon = filt.filter( vies );
+  assertEqual( schoon, "\'vies\' en \'smerig\' en \'apart\'" );
+  UniFilter filt3;
+  assertNoThrow( filt3.fill( path + "quotes.old.filter") );
+  schoon = filt.filter( vies );
+  assertEqual( schoon, "\'vies\' en \'smerig\' en \'apart\'" );
+  UniFilter filt4;
+  assertNoThrow( filt4.init( "‘ > \\' ;", "quote_filter" ) );
+  assertNoThrow( filt4.add( string("’ \' ") ) );
+  assertNoThrow( filt4.add( string("` \'") ) );
+  assertNoThrow( filt4.add( string("´ \' ") ) );
+  schoon = filt.filter( vies );
+  assertEqual( schoon, "\'vies\' en \'smerig\' en \'apart\'" );
+  UniFilter filt5;
+  assertNoThrow( filt5.init( "[:Hyphen:]+ > '-'; [:Dash:]+ > '-';",
+			     "hypen_filter" ) );
+  vies = "em—dash, en–dash, bar―, bar―――, 3em⸻dash, FullWidth－HyphenMinus,"
+    " 3FullWidth－－－HyphenMinus, vertical︱Emdash, hyp-en, 2hyp--ens";
+  schoon = filt5.filter( vies );
+  assertEqual( schoon, "em-dash, en-dash, bar-, bar-, 3em-dash, "
+	       "FullWidth-HyphenMinus, 3FullWidth-HyphenMinus, vertical-Emdash,"
+	       " hyp-en, 2hyp-ens" );
+  UniFilter filt6;
+  assertNoThrow( filt6.init( "^[:White_Space:]+ > ; [:White_Space:]+ > ' ';", "sep_filter" ) );
+  vies = " \t\t  Jan    en    Kees, \tKlaas\t \ten    Mies";
+  schoon = filt6.filter( vies );
+  assertEqual( schoon, "Jan en Kees, Klaas en Mies" );
+  assertEqual( filter_diacritics( "een appél is geen appèl" ), "een appel is geen appel" );
+  assertEqual( filter_diacritics( "de reeën zijn reeël" ), "de reeen zijn reeel" );
+}
+
 int main( const int argc, const char* argv[] ){
   cerr << BuildInfo() << endl;
   test_opts_basic();
@@ -565,20 +815,22 @@ int main( const int argc, const char* argv[] ){
   test_to_lower();
   test_uppercase();
   test_lowercase();
+  test_treehash();
+  test_lexicon();
+  test_realpath();
   string testdir;
   bool dummy;
   opts1.is_present( 'd', testdir, dummy );
-#ifdef HAVE_BZLIB_H
   test_bz2compression( testdir );
-#endif
-#ifdef HAVE_LIBZ
   test_gzcompression( testdir );
-#endif
-#ifdef HAVE_LIBTAR_H
   test_tar( testdir );
-#endif
+  test_base_dir();
   test_fileutils( testdir );
   test_configuration( testdir );
   test_logstream( testdir );
-  summarize_tests(3);
+  test_unicode( testdir );
+  test_unicode_split();
+  test_unicode_regex();
+  test_unicode_filters( testdir );
+  summarize_tests(4);
 }
